@@ -22,10 +22,18 @@
 from telegram import TelegramObject, ChatPhoto
 from .chatpermissions import ChatPermissions
 
+from telegram.utils.types import JSONDict
+from typing import Any, Optional, List, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot, Message, ChatMember
+
 
 class Chat(TelegramObject):
     """This object represents a chat.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`id` is equal.
+
     Attributes:
         id (:obj:`int`): Unique identifier for this chat.
         type (:obj:`str`): Type of chat.
@@ -37,11 +45,14 @@ class Chat(TelegramObject):
         description (:obj:`str`): Optional. Description, for groups, supergroups and channel chats.
         invite_link (:obj:`str`): Optional. Chat invite link, for supergroups and channel chats.
         pinned_message (:class:`telegram.Message`): Optional. Pinned message, for supergroups.
-            Returned only in get_chat.
-        permissions (:class:`telegram.ChatPermission`): Optional. Default chat member permissions,
-            for groups and supergroups. Returned only in getChat.
+            Returned only in :meth:`telegram.Bot.get_chat`.
+        permissions (:class:`telegram.ChatPermissions`): Optional. Default chat member permissions,
+            for groups and supergroups. Returned only in :meth:`telegram.Bot.get_chat`.
+        slow_mode_delay (:obj:`int`): Optional. For supergroups, the minimum allowed delay between
+            consecutive messages sent by each unprivileged user. Returned only in
+            :meth:`telegram.Bot.get_chat`.
         sticker_set_name (:obj:`str`): Optional. For supergroups, name of Group sticker set.
-        can_set_sticker_set (:obj:`bool`): Optional. ``True``, if the bot can change group the
+        can_set_sticker_set (:obj:`bool`): Optional. :obj:`True`, if the bot can change group the
             sticker set.
 
     Args:
@@ -56,49 +67,56 @@ class Chat(TelegramObject):
             available.
         first_name(:obj:`str`, optional): First name of the other party in a private chat.
         last_name(:obj:`str`, optional): Last name of the other party in a private chat.
-        photo (:class:`telegram.ChatPhoto`, optional): Chat photo. Returned only in getChat.
+        photo (:class:`telegram.ChatPhoto`, optional): Chat photo.
+            Returned only in :meth:`telegram.Bot.get_chat`.
         description (:obj:`str`, optional): Description, for groups, supergroups and channel chats.
-            Returned only in get_chat.
-        invite_link (:obj:`str`, optional): Chat invite link, for supergroups and channel chats.
-            Returned only in get_chat.
-        pinned_message (:class:`telegram.Message`, optional): Pinned message, for supergroups.
-            Returned only in get_chat.
-        permissions (:class:`telegram.ChatPermission`): Optional. Default chat member permissions,
-            for groups and supergroups. Returned only in getChat.
+            Returned only in :meth:`telegram.Bot.get_chat`.
+        invite_link (:obj:`str`, optional): Chat invite link, for groups, supergroups and channel
+            chats. Each administrator in a chat generates their own invite links, so the bot must
+            first generate the link using ``export_chat_invite_link()``. Returned only
+            in :meth:`telegram.Bot.get_chat`.
+        pinned_message (:class:`telegram.Message`, optional): Pinned message, for groups,
+            supergroups and channels. Returned only in :meth:`telegram.Bot.get_chat`.
+        permissions (:class:`telegram.ChatPermissions`): Optional. Default chat member permissions,
+            for groups and supergroups. Returned only in :meth:`telegram.Bot.get_chat`.
+        slow_mode_delay (:obj:`int`, optional): For supergroups, the minimum allowed delay between
+            consecutive messages sent by each unprivileged user.
+            Returned only in :meth:`telegram.Bot.get_chat`.
         bot (:class:`telegram.Bot`, optional): The Bot to use for instance methods.
-        sticker_set_name (:obj:`str`, optional): For supergroups, name of Group sticker set.
-            Returned only in get_chat.
-        can_set_sticker_set (:obj:`bool`, optional): ``True``, if the bot can change group the
-            sticker set. Returned only in get_chat.
+        sticker_set_name (:obj:`str`, optional): For supergroups, name of group sticker set.
+            Returned only in :meth:`telegram.Bot.get_chat`.
+        can_set_sticker_set (:obj:`bool`, optional): :obj:`True`, if the bot can change group the
+            sticker set. Returned only in :meth:`telegram.Bot.get_chat`.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
     """
 
-    PRIVATE = 'private'
+    PRIVATE: str = 'private'
     """:obj:`str`: 'private'"""
-    GROUP = 'group'
+    GROUP: str = 'group'
     """:obj:`str`: 'group'"""
-    SUPERGROUP = 'supergroup'
+    SUPERGROUP: str = 'supergroup'
     """:obj:`str`: 'supergroup'"""
-    CHANNEL = 'channel'
+    CHANNEL: str = 'channel'
     """:obj:`str`: 'channel'"""
 
     def __init__(self,
-                 id,
-                 type,
-                 title=None,
-                 username=None,
-                 first_name=None,
-                 last_name=None,
-                 bot=None,
-                 photo=None,
-                 description=None,
-                 invite_link=None,
-                 pinned_message=None,
-                 permissions=None,
-                 sticker_set_name=None,
-                 can_set_sticker_set=None,
-                 **kwargs):
+                 id: int,
+                 type: str,
+                 title: str = None,
+                 username: str = None,
+                 first_name: str = None,
+                 last_name: str = None,
+                 bot: 'Bot' = None,
+                 photo: ChatPhoto = None,
+                 description: str = None,
+                 invite_link: str = None,
+                 pinned_message: 'Message' = None,
+                 permissions: ChatPermissions = None,
+                 sticker_set_name: str = None,
+                 can_set_sticker_set: bool = None,
+                 slow_mode_delay: int = None,
+                 **kwargs: Any):
         # Required
         self.id = int(id)
         self.type = type
@@ -114,6 +132,7 @@ class Chat(TelegramObject):
         self.invite_link = invite_link
         self.pinned_message = pinned_message
         self.permissions = permissions
+        self.slow_mode_delay = slow_mode_delay
         self.sticker_set_name = sticker_set_name
         self.can_set_sticker_set = can_set_sticker_set
 
@@ -121,7 +140,7 @@ class Chat(TelegramObject):
         self._id_attrs = (self.id,)
 
     @property
-    def link(self):
+    def link(self) -> Optional[str]:
         """:obj:`str`: Convenience property. If the chat has a :attr:`username`, returns a t.me
         link of the chat."""
         if self.username:
@@ -129,36 +148,23 @@ class Chat(TelegramObject):
         return None
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: JSONDict, bot: 'Bot') -> Optional['Chat']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
         data['photo'] = ChatPhoto.de_json(data.get('photo'), bot)
         from telegram import Message
-        pinned_message = data.get('pinned_message')
-        if pinned_message:
-            pinned_message['default_quote'] = data.get('default_quote')
-        data['pinned_message'] = Message.de_json(pinned_message, bot)
+        data['pinned_message'] = Message.de_json(data.get('pinned_message'), bot)
         data['permissions'] = ChatPermissions.de_json(data.get('permissions'), bot)
 
         return cls(bot=bot, **data)
 
-    def send_action(self, *args, **kwargs):
+    def leave(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
-            bot.send_chat_action(update.message.chat.id, *args, **kwargs)
-
-        Returns:
-            :obj:`bool`: If the action was sent successfully.
-
-        """
-
-        return self.bot.send_chat_action(self.id, *args, **kwargs)
-
-    def leave(self, *args, **kwargs):
-        """Shortcut for::
-
-            bot.leave_chat(update.message.chat.id, *args, **kwargs)
+            bot.leave_chat(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool` If the action was sent successfully.
@@ -166,24 +172,24 @@ class Chat(TelegramObject):
         """
         return self.bot.leave_chat(self.id, *args, **kwargs)
 
-    def get_administrators(self, *args, **kwargs):
+    def get_administrators(self, *args: Any, **kwargs: Any) -> List['ChatMember']:
         """Shortcut for::
 
-            bot.get_chat_administrators(update.message.chat.id, *args, **kwargs)
+            bot.get_chat_administrators(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             List[:class:`telegram.ChatMember`]: A list of administrators in a chat. An Array of
             :class:`telegram.ChatMember` objects that contains information about all
             chat administrators except other bots. If the chat is a group or a supergroup
-            and no administrators were appointed, only the creator will be returned
+            and no administrators were appointed, only the creator will be returned.
 
         """
         return self.bot.get_chat_administrators(self.id, *args, **kwargs)
 
-    def get_members_count(self, *args, **kwargs):
+    def get_members_count(self, *args: Any, **kwargs: Any) -> int:
         """Shortcut for::
 
-            bot.get_chat_members_count(update.message.chat.id, *args, **kwargs)
+            bot.get_chat_members_count(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :obj:`int`
@@ -191,10 +197,10 @@ class Chat(TelegramObject):
         """
         return self.bot.get_chat_members_count(self.id, *args, **kwargs)
 
-    def get_member(self, *args, **kwargs):
+    def get_member(self, *args: Any, **kwargs: Any) -> 'ChatMember':
         """Shortcut for::
 
-            bot.get_chat_member(update.message.chat.id, *args, **kwargs)
+            bot.get_chat_member(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.ChatMember`
@@ -202,13 +208,13 @@ class Chat(TelegramObject):
         """
         return self.bot.get_chat_member(self.id, *args, **kwargs)
 
-    def kick_member(self, *args, **kwargs):
+    def kick_member(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
-                bot.kick_chat_member(update.message.chat.id, *args, **kwargs)
+                bot.kick_chat_member(update.effective_chat.id, *args, **kwargs)
 
         Returns:
-            :obj:`bool`: If the action was sent succesfully.
+            :obj:`bool`: If the action was sent successfully.
 
         Note:
             This method will only work if the `All Members Are Admins` setting is off in the
@@ -218,10 +224,10 @@ class Chat(TelegramObject):
         """
         return self.bot.kick_chat_member(self.id, *args, **kwargs)
 
-    def unban_member(self, *args, **kwargs):
+    def unban_member(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
-                bot.unban_chat_member(update.message.chat.id, *args, **kwargs)
+                bot.unban_chat_member(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :obj:`bool`: If the action was sent successfully.
@@ -229,23 +235,32 @@ class Chat(TelegramObject):
         """
         return self.bot.unban_chat_member(self.id, *args, **kwargs)
 
-    def set_permissions(self, *args, **kwargs):
+    def set_permissions(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
-                bot.set_chat_permissions(update.message.chat.id, *args, **kwargs)
+                bot.set_chat_permissions(update.effective_chat.id, *args, **kwargs)
+
+        Returns:
+            :obj:`bool`: If the action was sent successfully.
+
+    """
+        return self.bot.set_chat_permissions(self.id, *args, **kwargs)
+
+    def set_administrator_custom_title(self, *args: Any, **kwargs: Any) -> bool:
+        """Shortcut for::
+
+                bot.set_chat_administrator_custom_title(update.effective_chat.id, *args, **kwargs)
 
         Returns:
         :obj:`bool`: If the action was sent successfully.
 
     """
-        return self.bot.set_chat_permissions(self.id, *args, **kwargs)
+        return self.bot.set_chat_administrator_custom_title(self.id, *args, **kwargs)
 
-    def send_message(self, *args, **kwargs):
+    def send_message(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_message(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_message(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -253,12 +268,35 @@ class Chat(TelegramObject):
         """
         return self.bot.send_message(self.id, *args, **kwargs)
 
-    def send_photo(self, *args, **kwargs):
+    def send_media_group(self, *args: Any, **kwargs: Any) -> List['Message']:
         """Shortcut for::
 
-            bot.send_photo(Chat.id, *args, **kwargs)
+            bot.send_media_group(update.effective_chat.id, *args, **kwargs)
 
-        Where Chat is the current instance.
+        Returns:
+            List[:class:`telegram.Message`:] On success, instance representing the message posted.
+
+        """
+        return self.bot.send_media_group(self.id, *args, **kwargs)
+
+    def send_chat_action(self, *args: Any, **kwargs: Any) -> bool:
+        """Shortcut for::
+
+            bot.send_chat_action(update.effective_chat.id, *args, **kwargs)
+
+        Returns:
+            :obj:`True`: On success.
+
+        """
+        return self.bot.send_chat_action(self.id, *args, **kwargs)
+
+    send_action = send_chat_action
+    """Alias for :attr:`send_chat_action`"""
+
+    def send_photo(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_photo(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -266,12 +304,21 @@ class Chat(TelegramObject):
         """
         return self.bot.send_photo(self.id, *args, **kwargs)
 
-    def send_audio(self, *args, **kwargs):
+    def send_contact(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_audio(Chat.id, *args, **kwargs)
+            bot.send_contact(update.effective_chat.id, *args, **kwargs)
 
-        Where Chat is the current instance.
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_contact(self.id, *args, **kwargs)
+
+    def send_audio(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_audio(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -279,12 +326,10 @@ class Chat(TelegramObject):
         """
         return self.bot.send_audio(self.id, *args, **kwargs)
 
-    def send_document(self, *args, **kwargs):
+    def send_document(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_document(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_document(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -292,12 +337,54 @@ class Chat(TelegramObject):
         """
         return self.bot.send_document(self.id, *args, **kwargs)
 
-    def send_animation(self, *args, **kwargs):
+    def send_dice(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_animation(Chat.id, *args, **kwargs)
+            bot.send_dice(update.effective_chat.id, *args, **kwargs)
 
-        Where Chat is the current instance.
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_dice(self.id, *args, **kwargs)
+
+    def send_game(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_game(update.effective_chat.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_game(self.id, *args, **kwargs)
+
+    def send_invoice(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_invoice(update.effective_chat.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_invoice(self.id, *args, **kwargs)
+
+    def send_location(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_location(update.effective_chat.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_location(self.id, *args, **kwargs)
+
+    def send_animation(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_animation(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -305,12 +392,10 @@ class Chat(TelegramObject):
         """
         return self.bot.send_animation(self.id, *args, **kwargs)
 
-    def send_sticker(self, *args, **kwargs):
+    def send_sticker(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_sticker(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_sticker(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -318,12 +403,21 @@ class Chat(TelegramObject):
         """
         return self.bot.send_sticker(self.id, *args, **kwargs)
 
-    def send_video(self, *args, **kwargs):
+    def send_venue(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_video(Chat.id, *args, **kwargs)
+            bot.send_venue(update.effective_chat.id, *args, **kwargs)
 
-        Where Chat is the current instance.
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_venue(self.id, *args, **kwargs)
+
+    def send_video(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_video(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -331,12 +425,10 @@ class Chat(TelegramObject):
         """
         return self.bot.send_video(self.id, *args, **kwargs)
 
-    def send_video_note(self, *args, **kwargs):
+    def send_video_note(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_video_note(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_video_note(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -344,12 +436,10 @@ class Chat(TelegramObject):
         """
         return self.bot.send_video_note(self.id, *args, **kwargs)
 
-    def send_voice(self, *args, **kwargs):
+    def send_voice(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_voice(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_voice(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -357,12 +447,10 @@ class Chat(TelegramObject):
         """
         return self.bot.send_voice(self.id, *args, **kwargs)
 
-    def send_poll(self, *args, **kwargs):
+    def send_poll(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_poll(Chat.id, *args, **kwargs)
-
-        Where Chat is the current instance.
+            bot.send_poll(update.effective_chat.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
