@@ -17,13 +17,22 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram ChatMember."""
+import datetime
 
 from telegram import User, TelegramObject
 from telegram.utils.helpers import to_timestamp, from_timestamp
 
+from telegram.utils.types import JSONDict
+from typing import Any, Optional, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot
+
 
 class ChatMember(TelegramObject):
-    """This object contains information about one member of the chat.
+    """This object contains information about one member of a chat.
+
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`user` and :attr:`status` are equal.
 
     Attributes:
         user (:class:`telegram.User`): Information about the user.
@@ -46,13 +55,13 @@ class ChatMember(TelegramObject):
         can_pin_messages (:obj:`bool`): Optional. If the user can pin messages.
         can_promote_members (:obj:`bool`): Optional. If the administrator can add new
             administrators.
-        is_member (:obj:`bool`): Optional. Restricted only. True, if the user is a member of the
-            chat at the moment of the request.
+        is_member (:obj:`bool`): Optional. Restricted only. :obj:`True`, if the user is a member of
+            the chat at the moment of the request.
         can_send_messages (:obj:`bool`): Optional. If the user can send text messages, contacts,
             locations and venues.
         can_send_media_messages (:obj:`bool`): Optional. If the user can send media messages,
             implies can_send_messages.
-        can_send_polls (:obj:`bool`): Optional. True, if the user is allowed to
+        can_send_polls (:obj:`bool`): Optional. :obj:`True`, if the user is allowed to
             send polls.
         can_send_other_messages (:obj:`bool`): Optional. If the user can send animations, games,
             stickers and use inline bots, implies can_send_media_messages.
@@ -67,61 +76,74 @@ class ChatMember(TelegramObject):
             Custom title for this user.
         until_date (:class:`datetime.datetime`, optional): Restricted and kicked only. Date when
             restrictions will be lifted for this user.
-        can_be_edited (:obj:`bool`, optional): Administrators only. True, if the bot is allowed to
-            edit administrator privileges of that user.
-        can_change_info (:obj:`bool`, optional): Administrators and restricted only. True, if the
-            user can change the chat title, photo and other settings.
-        can_post_messages (:obj:`bool`, optional): Administrators only. True, if the administrator
-            can post in the channel, channels only.
-        can_edit_messages (:obj:`bool`, optional): Administrators only. True, if the administrator
-            can edit messages of other users, channels only.
-        can_delete_messages (:obj:`bool`, optional): Administrators only. True, if the
-            administrator can delete messages of other user.
-        can_invite_users (:obj:`bool`, optional): Administrators and restricted only. True, if the
-            user can invite new users to the chat.
-        can_restrict_members (:obj:`bool`, optional): Administrators only. True, if the
+        can_be_edited (:obj:`bool`, optional): Administrators only. :obj:`True`, if the bot is
+            allowed to edit administrator privileges of that user.
+        can_change_info (:obj:`bool`, optional): Administrators and restricted only. :obj:`True`,
+            if the user can change the chat title, photo and other settings.
+        can_post_messages (:obj:`bool`, optional): Administrators only. :obj:`True`, if the
+            administrator can post in the channel, channels only.
+        can_edit_messages (:obj:`bool`, optional): Administrators only. :obj:`True`, if the
+            administrator can edit messages of other users and can pin messages; channels only.
+        can_delete_messages (:obj:`bool`, optional): Administrators only. :obj:`True`, if the
+            administrator can delete messages of other users.
+        can_invite_users (:obj:`bool`, optional): Administrators and restricted only. :obj:`True`,
+            if the user can invite new users to the chat.
+        can_restrict_members (:obj:`bool`, optional): Administrators only. :obj:`True`, if the
             administrator can restrict, ban or unban chat members.
-        can_pin_messages (:obj:`bool`, optional): Administrators and restricted only. True, if the
-            user can pin messages, supergroups only.
-        can_promote_members (:obj:`bool`, optional): Administrators only. True, if the
+        can_pin_messages (:obj:`bool`, optional): Administrators and restricted only. :obj:`True`,
+            if the user can pin messages, groups and supergroups only.
+        can_promote_members (:obj:`bool`, optional): Administrators only. :obj:`True`, if the
             administrator can add new administrators with a subset of his own privileges or demote
             administrators that he has promoted, directly or indirectly (promoted by administrators
             that were appointed by the user).
-        is_member (:obj:`bool`, optional): Restricted only. True, if the user is a member of the
-            chat at the moment of the request.
-        can_send_messages (:obj:`bool`, optional): Restricted only. True, if the user can send text
-            messages, contacts, locations and venues.
-        can_send_media_messages (:obj:`bool`, optional): Restricted only. True, if the user can
-            send audios, documents, photos, videos, video notes and voice notes, implies
-            can_send_messages.
-        can_send_polls (:obj:`bool`, optional): Restricted only. True, if the user is allowed to
-            send polls.
-        can_send_other_messages (:obj:`bool`, optional): Restricted only. True, if the user can
-            send animations, games, stickers and use inline bots, implies can_send_media_messages.
-        can_add_web_page_previews (:obj:`bool`, optional): Restricted only. True, if user may add
-            web page previews to his messages, implies can_send_media_messages.
+        is_member (:obj:`bool`, optional): Restricted only. :obj:`True`, if the user is a member of
+            the chat at the moment of the request.
+        can_send_messages (:obj:`bool`, optional): Restricted only. :obj:`True`, if the user can
+            send text messages, contacts, locations and venues.
+        can_send_media_messages (:obj:`bool`, optional): Restricted only. :obj:`True`, if the user
+            can send audios, documents, photos, videos, video notes and voice notes.
+        can_send_polls (:obj:`bool`, optional): Restricted only. :obj:`True`, if the user is
+            allowed to send polls.
+        can_send_other_messages (:obj:`bool`, optional): Restricted only. :obj:`True`, if the user
+            can send animations, games, stickers and use inline bots.
+        can_add_web_page_previews (:obj:`bool`, optional): Restricted only. :obj:`True`, if user
+            may add web page previews to his messages.
 
     """
-    ADMINISTRATOR = 'administrator'
+    ADMINISTRATOR: str = 'administrator'
     """:obj:`str`: 'administrator'"""
-    CREATOR = 'creator'
+    CREATOR: str = 'creator'
     """:obj:`str`: 'creator'"""
-    KICKED = 'kicked'
+    KICKED: str = 'kicked'
     """:obj:`str`: 'kicked'"""
-    LEFT = 'left'
+    LEFT: str = 'left'
     """:obj:`str`: 'left'"""
-    MEMBER = 'member'
+    MEMBER: str = 'member'
     """:obj:`str`: 'member'"""
-    RESTRICTED = 'restricted'
+    RESTRICTED: str = 'restricted'
     """:obj:`str`: 'restricted'"""
 
-    def __init__(self, user, status, until_date=None, can_be_edited=None,
-                 can_change_info=None, can_post_messages=None, can_edit_messages=None,
-                 can_delete_messages=None, can_invite_users=None,
-                 can_restrict_members=None, can_pin_messages=None,
-                 can_promote_members=None, can_send_messages=None,
-                 can_send_media_messages=None, can_send_polls=None, can_send_other_messages=None,
-                 can_add_web_page_previews=None, is_member=None, custom_title=None, **kwargs):
+    def __init__(self,
+                 user: User,
+                 status: str,
+                 until_date: datetime.datetime = None,
+                 can_be_edited: bool = None,
+                 can_change_info: bool = None,
+                 can_post_messages: bool = None,
+                 can_edit_messages: bool = None,
+                 can_delete_messages: bool = None,
+                 can_invite_users: bool = None,
+                 can_restrict_members: bool = None,
+                 can_pin_messages: bool = None,
+                 can_promote_members: bool = None,
+                 can_send_messages: bool = None,
+                 can_send_media_messages: bool = None,
+                 can_send_polls: bool = None,
+                 can_send_other_messages: bool = None,
+                 can_add_web_page_previews: bool = None,
+                 is_member: bool = None,
+                 custom_title: str = None,
+                 **kwargs: Any):
         # Required
         self.user = user
         self.status = status
@@ -146,19 +168,19 @@ class ChatMember(TelegramObject):
         self._id_attrs = (self.user, self.status)
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['ChatMember']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super(ChatMember, cls).de_json(data, bot)
-
         data['user'] = User.de_json(data.get('user'), bot)
         data['until_date'] = from_timestamp(data.get('until_date', None))
 
         return cls(**data)
 
-    def to_dict(self):
-        data = super(ChatMember, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         data['until_date'] = to_timestamp(self.until_date)
 
