@@ -21,12 +21,19 @@
 import sys
 
 from telegram import MessageEntity, TelegramObject, Animation, PhotoSize
+from telegram.utils.types import JSONDict
+from typing import List, Any, Dict, Optional, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class Game(TelegramObject):
     """
-    This object represents a game. Use BotFather to create and edit games, their short names will
-    act as unique identifiers.
+    This object represents a game. Use `BotFather <https://t.me/BotFather>`_ to create and edit
+    games, their short names will act as unique identifiers.
+
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`title`, :attr:`description` and :attr:`photo` are equal.
 
     Attributes:
         title (:obj:`str`): Title of the game.
@@ -35,11 +42,12 @@ class Game(TelegramObject):
             in chats.
         text (:obj:`str`): Optional. Brief description of the game or high scores included in the
             game message. Can be automatically edited to include current high scores for the game
-            when the bot calls set_game_score, or manually edited using edit_message_text.
+            when the bot calls :meth:`telegram.Bot.set_game_score`, or manually edited
+            using :meth:`telegram.Bot.edit_message_text`.
         text_entities (List[:class:`telegram.MessageEntity`]): Optional. Special entities that
             appear in text, such as usernames, URLs, bot commands, etc.
         animation (:class:`telegram.Animation`): Optional. Animation that will be displayed in the
-            game message in chats. Upload via BotFather.
+            game message in chats. Upload via `BotFather <https://t.me/BotFather>`_.
 
     Args:
         title (:obj:`str`): Title of the game.
@@ -48,44 +56,49 @@ class Game(TelegramObject):
             in chats.
         text (:obj:`str`, optional): Brief description of the game or high scores included in the
             game message. Can be automatically edited to include current high scores for the game
-            when the bot calls set_game_score, or manually edited using edit_message_text.
-            0-4096 characters. Also found as ``telegram.constants.MAX_MESSAGE_LENGTH``.
+            when the bot calls :meth:`telegram.Bot.set_game_score`, or manually edited
+            using :meth:`telegram.Bot.edit_message_text`.
+            1-4096 characters. Also found as ``telegram.constants.MAX_MESSAGE_LENGTH``.
         text_entities (List[:class:`telegram.MessageEntity`], optional): Special entities that
             appear in text, such as usernames, URLs, bot commands, etc.
         animation (:class:`telegram.Animation`, optional): Animation that will be displayed in the
-            game message in chats. Upload via BotFather.
+            game message in chats. Upload via `BotFather <https://t.me/BotFather>`_.
 
     """
 
     def __init__(self,
-                 title,
-                 description,
-                 photo,
-                 text=None,
-                 text_entities=None,
-                 animation=None,
-                 **kwargs):
+                 title: str,
+                 description: str,
+                 photo: List[PhotoSize],
+                 text: str = None,
+                 text_entities: List[MessageEntity] = None,
+                 animation: Animation = None,
+                 **kwargs: Any):
+        # Required
         self.title = title
         self.description = description
         self.photo = photo
+        # Optionals
         self.text = text
         self.text_entities = text_entities or list()
         self.animation = animation
 
+        self._id_attrs = (self.title, self.description, self.photo)
+
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['Game']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super().de_json(data, bot)
-
         data['photo'] = PhotoSize.de_list(data.get('photo'), bot)
         data['text_entities'] = MessageEntity.de_list(data.get('text_entities'), bot)
         data['animation'] = Animation.de_json(data.get('animation'), bot)
 
         return cls(**data)
 
-    def to_dict(self):
+    def to_dict(self) -> JSONDict:
         data = super().to_dict()
 
         data['photo'] = [p.to_dict() for p in self.photo]
@@ -94,7 +107,7 @@ class Game(TelegramObject):
 
         return data
 
-    def parse_text_entity(self, entity):
+    def parse_text_entity(self, entity: MessageEntity) -> str:
         """Returns the text from a given :class:`telegram.MessageEntity`.
 
         Note:
@@ -109,7 +122,13 @@ class Game(TelegramObject):
         Returns:
             :obj:`str`: The text of the given entity.
 
+        Raises:
+            RuntimeError: If this game has no text.
+
         """
+        if not self.text:
+            raise RuntimeError("This Game has no 'text'.")
+
         # Is it a narrow build, if so we don't need to convert
         if sys.maxunicode == 0xffff:
             return self.text[entity.offset:entity.offset + entity.length]
@@ -119,7 +138,7 @@ class Game(TelegramObject):
 
         return entity_text.decode('utf-16-le')
 
-    def parse_text_entities(self, types=None):
+    def parse_text_entities(self, types: List[str] = None) -> Dict[MessageEntity, str]:
         """
         Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
         It contains entities from this message filtered by their ``type`` attribute as the key, and
@@ -145,5 +164,8 @@ class Game(TelegramObject):
 
         return {
             entity: self.parse_text_entity(entity)
-            for entity in self.text_entities if entity.type in types
+            for entity in (self.text_entities or []) if entity.type in types
         }
+
+    def __hash__(self) -> int:
+        return hash((self.title, self.description, tuple(p for p in self.photo)))
