@@ -19,7 +19,10 @@
 """This module contains the classes that represent Telegram InlineQueryResultAudio."""
 
 from telegram import InlineQueryResult
-from telegram.utils.helpers import DEFAULT_NONE
+from telegram.utils.helpers import DEFAULT_NONE, DefaultValue
+from typing import Any, Union, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import InputMessageContent, ReplyMarkup
 
 
 class InlineQueryResultAudio(InlineQueryResult):
@@ -63,19 +66,19 @@ class InlineQueryResultAudio(InlineQueryResult):
     """
 
     def __init__(self,
-                 id,
-                 audio_url,
-                 title,
-                 performer=None,
-                 audio_duration=None,
-                 caption=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 parse_mode=DEFAULT_NONE,
-                 **kwargs):
+                 id: str,
+                 audio_url: str,
+                 title: str,
+                 performer: str = None,
+                 audio_duration: int = None,
+                 caption: str = None,
+                 reply_markup: 'ReplyMarkup' = None,
+                 input_message_content: 'InputMessageContent' = None,
+                 parse_mode: Union[str, DefaultValue] = DEFAULT_NONE,
+                 **kwargs: Any):
 
         # Required
-        super(InlineQueryResultAudio, self).__init__('audio', id)
+        super().__init__('audio', id)
         self.audio_url = audio_url
         self.title = title
 
