@@ -19,6 +19,9 @@
 """This module contains the classes that represent Telegram InlineQueryResultCachedSticker."""
 
 from telegram import InlineQueryResult
+from typing import Any, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import ReplyMarkup, InputMessageContent
 
 
 class InlineQueryResultCachedSticker(InlineQueryResult):
@@ -48,13 +51,13 @@ class InlineQueryResultCachedSticker(InlineQueryResult):
     """
 
     def __init__(self,
-                 id,
-                 sticker_file_id,
-                 reply_markup=None,
-                 input_message_content=None,
-                 **kwargs):
+                 id: str,
+                 sticker_file_id: str,
+                 reply_markup: 'ReplyMarkup' = None,
+                 input_message_content: 'InputMessageContent' = None,
+                 **kwargs: Any):
         # Required
-        super(InlineQueryResultCachedSticker, self).__init__('sticker', id)
+        super().__init__('sticker', id)
         self.sticker_file_id = sticker_file_id
 
         # Optionals
