@@ -19,7 +19,10 @@
 """This module contains the classes that represent Telegram InlineQueryResultGif."""
 
 from telegram import InlineQueryResult
-from telegram.utils.helpers import DEFAULT_NONE
+from telegram.utils.helpers import DEFAULT_NONE, DefaultValue
+from typing import Any, Union, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import InputMessageContent, ReplyMarkup
 
 
 class InlineQueryResultGif(InlineQueryResult):
@@ -35,7 +38,9 @@ class InlineQueryResultGif(InlineQueryResult):
         gif_width (:obj:`int`): Optional. Width of the GIF.
         gif_height (:obj:`int`): Optional. Height of the GIF.
         gif_duration (:obj:`int`): Optional. Duration of the GIF.
-        thumb_url (:obj:`str`): URL of the static thumbnail for the result (jpeg or gif).
+        thumb_url (:obj:`str`): URL of the static (JPEG or GIF) or animated (MPEG4) thumbnail for
+            the result.
+        thumb_mime_type (:obj:`str`): Optional. MIME type of the thumbnail.
         title (:obj:`str`): Optional. Title for the result.
         caption (:obj:`str`): Optional. Caption of the GIF file to be sent, 0-1024 characters
             after entities parsing.
@@ -53,7 +58,10 @@ class InlineQueryResultGif(InlineQueryResult):
         gif_width (:obj:`int`, optional): Width of the GIF.
         gif_height (:obj:`int`, optional): Height of the GIF.
         gif_duration (:obj:`int`, optional): Duration of the GIF
-        thumb_url (:obj:`str`): URL of the static thumbnail for the result (jpeg or gif).
+        thumb_url (:obj:`str`): URL of the static (JPEG or GIF) or animated (MPEG4) thumbnail for
+            the result.
+        thumb_mime_type (:obj:`str`, optional): MIME type of the thumbnail, must be one of
+            ``'image/jpeg'``, ``'image/gif'``, or ``'video/mp4'``. Defaults to ``'image/jpeg'``.
         title (:obj:`str`, optional): Title for the result.
         caption (:obj:`str`, optional): Caption of the GIF file to be sent, 0-1024 characters
             after entities parsing.
@@ -69,21 +77,22 @@ class InlineQueryResultGif(InlineQueryResult):
     """
 
     def __init__(self,
-                 id,
-                 gif_url,
-                 thumb_url,
-                 gif_width=None,
-                 gif_height=None,
-                 title=None,
-                 caption=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 gif_duration=None,
-                 parse_mode=DEFAULT_NONE,
-                 **kwargs):
+                 id: str,
+                 gif_url: str,
+                 thumb_url: str,
+                 gif_width: int = None,
+                 gif_height: int = None,
+                 title: str = None,
+                 caption: str = None,
+                 reply_markup: 'ReplyMarkup' = None,
+                 input_message_content: 'InputMessageContent' = None,
+                 gif_duration: int = None,
+                 parse_mode: Union[str, DefaultValue] = DEFAULT_NONE,
+                 thumb_mime_type: str = None,
+                 **kwargs: Any):
 
         # Required
-        super(InlineQueryResultGif, self).__init__('gif', id)
+        super().__init__('gif', id)
         self.gif_url = gif_url
         self.thumb_url = thumb_url
 
@@ -96,3 +105,4 @@ class InlineQueryResultGif(InlineQueryResult):
         self.parse_mode = parse_mode
         self.reply_markup = reply_markup
         self.input_message_content = input_message_content
+        self.thumb_mime_type = thumb_mime_type
