@@ -19,6 +19,9 @@
 """This module contains the classes that represent Telegram InlineQueryResultLocation."""
 
 from telegram import InlineQueryResult
+from typing import Any, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import ReplyMarkup, InputMessageContent
 
 
 class InlineQueryResultLocation(InlineQueryResult):
@@ -62,19 +65,19 @@ class InlineQueryResultLocation(InlineQueryResult):
     """
 
     def __init__(self,
-                 id,
-                 latitude,
-                 longitude,
-                 title,
-                 live_period=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 thumb_url=None,
-                 thumb_width=None,
-                 thumb_height=None,
-                 **kwargs):
+                 id: str,
+                 latitude: float,
+                 longitude: float,
+                 title: str,
+                 live_period: int = None,
+                 reply_markup: 'ReplyMarkup' = None,
+                 input_message_content: 'InputMessageContent' = None,
+                 thumb_url: str = None,
+                 thumb_width: int = None,
+                 thumb_height: int = None,
+                 **kwargs: Any):
         # Required
-        super(InlineQueryResultLocation, self).__init__('location', id)
+        super().__init__('location', id)
         self.latitude = latitude
         self.longitude = longitude
         self.title = title
