@@ -19,7 +19,10 @@
 """This module contains the classes that represent Telegram InlineQueryResultVoice."""
 
 from telegram import InlineQueryResult
-from telegram.utils.helpers import DEFAULT_NONE
+from telegram.utils.helpers import DEFAULT_NONE, DefaultValue
+from typing import Any, Union, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import InputMessageContent, ReplyMarkup
 
 
 class InlineQueryResultVoice(InlineQueryResult):
@@ -62,18 +65,18 @@ class InlineQueryResultVoice(InlineQueryResult):
     """
 
     def __init__(self,
-                 id,
-                 voice_url,
-                 title,
-                 voice_duration=None,
-                 caption=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 parse_mode=DEFAULT_NONE,
-                 **kwargs):
+                 id: str,
+                 voice_url: str,
+                 title: str,
+                 voice_duration: int = None,
+                 caption: str = None,
+                 reply_markup: 'ReplyMarkup' = None,
+                 input_message_content: 'InputMessageContent' = None,
+                 parse_mode: Union[str, DefaultValue] = DEFAULT_NONE,
+                 **kwargs: Any):
 
         # Required
-        super(InlineQueryResultVoice, self).__init__('voice', id)
+        super().__init__('voice', id)
         self.voice_url = voice_url
         self.title = title
 
