@@ -19,11 +19,16 @@
 """This module contains the classes that represent Telegram InputVenueMessageContent."""
 
 from telegram import InputMessageContent
+from typing import Any
 
 
 class InputVenueMessageContent(InputMessageContent):
     """Represents the content of a venue message to be sent as the result of an inline query.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`latitude`, :attr:`longitude` and :attr:`title`
+    are equal.
+
     Attributes:
         latitude (:obj:`float`): Latitude of the location in degrees.
         longitude (:obj:`float`): Longitude of the location in degrees.
@@ -47,8 +52,14 @@ class InputVenueMessageContent(InputMessageContent):
 
     """
 
-    def __init__(self, latitude, longitude, title, address, foursquare_id=None,
-                 foursquare_type=None, **kwargs):
+    def __init__(self,
+                 latitude: float,
+                 longitude: float,
+                 title: str,
+                 address: str,
+                 foursquare_id: str = None,
+                 foursquare_type: str = None,
+                 **kwargs: Any):
         # Required
         self.latitude = latitude
         self.longitude = longitude
@@ -57,3 +68,9 @@ class InputVenueMessageContent(InputMessageContent):
         # Optionals
         self.foursquare_id = foursquare_id
         self.foursquare_type = foursquare_type
+
+        self._id_attrs = (
+            self.latitude,
+            self.longitude,
+            self.title,
+        )
