@@ -19,6 +19,11 @@
 """This module contains an object that represents a Telegram MessageEntity."""
 
 from telegram import User, TelegramObject
+from telegram.utils.types import JSONDict
+from typing import Any, Optional, List, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class MessageEntity(TelegramObject):
@@ -26,32 +31,41 @@ class MessageEntity(TelegramObject):
     This object represents one special entity in a text message. For example, hashtags,
     usernames, URLs, etc.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`type`, :attr:`offset` and :attr`length` are equal.
+
     Attributes:
         type (:obj:`str`): Type of the entity.
         offset (:obj:`int`): Offset in UTF-16 code units to the start of the entity.
         length (:obj:`int`): Length of the entity in UTF-16 code units.
         url (:obj:`str`): Optional. Url that will be opened after user taps on the text.
         user (:class:`telegram.User`): Optional. The mentioned user.
-        language (:obj:`str`): Optional. Programming language of the entity
-            text
+        language (:obj:`str`): Optional. Programming language of the entity text.
 
     Args:
         type (:obj:`str`): Type of the entity. Can be mention (@username), hashtag, bot_command,
-            url, email, bold (bold text), italic (italic text), code (monowidth string), pre
-            (monowidth block), text_link (for clickable text URLs), text_mention (for users
-            without usernames).
+            url, email, phone_number, bold (bold text), italic (italic text), strikethrough,
+            code (monowidth string), pre (monowidth block), text_link (for clickable text URLs),
+            text_mention (for users without usernames).
         offset (:obj:`int`): Offset in UTF-16 code units to the start of the entity.
         length (:obj:`int`): Length of the entity in UTF-16 code units.
         url (:obj:`str`, optional): For :attr:`TEXT_LINK` only, url that will be opened after
-            usertaps on the text.
+            user taps on the text.
         user (:class:`telegram.User`, optional): For :attr:`TEXT_MENTION` only, the mentioned
              user.
         language (:obj:`str`, optional): For :attr:`PRE` only, the programming language of
-            the entity text
+            the entity text.
 
     """
 
-    def __init__(self, type, offset, length, url=None, user=None, language=None, **kwargs):
+    def __init__(self,
+                 type: str,
+                 offset: int,
+                 length: int,
+                 url: str = None,
+                 user: User = None,
+                 language: str = None,
+                 **kwargs: Any):
         # Required
         self.type = type
         self.offset = offset
@@ -64,8 +78,8 @@ class MessageEntity(TelegramObject):
         self._id_attrs = (self.type, self.offset, self.length)
 
     @classmethod
-    def de_json(cls, data, bot):
-        data = super(MessageEntity, cls).de_json(data, bot)
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['MessageEntity']:
+        data = cls.parse_data(data)
 
         if not data:
             return None
@@ -74,48 +88,37 @@ class MessageEntity(TelegramObject):
 
         return cls(**data)
 
-    @classmethod
-    def de_list(cls, data, bot):
-        if not data:
-            return list()
-
-        entities = list()
-        for entity in data:
-            entities.append(cls.de_json(entity, bot))
-
-        return entities
-
-    MENTION = 'mention'
+    MENTION: str = 'mention'
     """:obj:`str`: 'mention'"""
-    HASHTAG = 'hashtag'
+    HASHTAG: str = 'hashtag'
     """:obj:`str`: 'hashtag'"""
-    CASHTAG = 'cashtag'
+    CASHTAG: str = 'cashtag'
     """:obj:`str`: 'cashtag'"""
-    PHONE_NUMBER = 'phone_number'
+    PHONE_NUMBER: str = 'phone_number'
     """:obj:`str`: 'phone_number'"""
-    BOT_COMMAND = 'bot_command'
+    BOT_COMMAND: str = 'bot_command'
     """:obj:`str`: 'bot_command'"""
-    URL = 'url'
+    URL: str = 'url'
     """:obj:`str`: 'url'"""
-    EMAIL = 'email'
+    EMAIL: str = 'email'
     """:obj:`str`: 'email'"""
-    BOLD = 'bold'
+    BOLD: str = 'bold'
     """:obj:`str`: 'bold'"""
-    ITALIC = 'italic'
+    ITALIC: str = 'italic'
     """:obj:`str`: 'italic'"""
-    CODE = 'code'
+    CODE: str = 'code'
     """:obj:`str`: 'code'"""
-    PRE = 'pre'
+    PRE: str = 'pre'
     """:obj:`str`: 'pre'"""
-    TEXT_LINK = 'text_link'
+    TEXT_LINK: str = 'text_link'
     """:obj:`str`: 'text_link'"""
-    TEXT_MENTION = 'text_mention'
+    TEXT_MENTION: str = 'text_mention'
     """:obj:`str`: 'text_mention'"""
-    UNDERLINE = 'underline'
+    UNDERLINE: str = 'underline'
     """:obj:`str`: 'underline'"""
-    STRIKETHROUGH = 'strikethrough'
+    STRIKETHROUGH: str = 'strikethrough'
     """:obj:`str`: 'strikethrough'"""
-    ALL_TYPES = [
+    ALL_TYPES: List[str] = [
         MENTION, HASHTAG, CASHTAG, PHONE_NUMBER, BOT_COMMAND, URL,
         EMAIL, BOLD, ITALIC, CODE, PRE, TEXT_LINK, TEXT_MENTION, UNDERLINE, STRIKETHROUGH
     ]
