@@ -20,12 +20,17 @@
 """This module contains an object that represents a Telegram Message Parse Modes."""
 
 
-class ParseMode(object):
+class ParseMode:
     """This object represents a Telegram Message Parse Modes."""
 
-    MARKDOWN = 'Markdown'
-    """:obj:`str`: 'Markdown'"""
-    MARKDOWN_V2 = 'MarkdownV2'
+    MARKDOWN: str = 'Markdown'
+    """:obj:`str`: 'Markdown'
+
+    Note:
+        :attr:`MARKDOWN` is a legacy mode, retained by Telegram for backward compatibility.
+        You should use :attr:`MARKDOWN_V2` instead.
+    """
+    MARKDOWN_V2: str = 'MarkdownV2'
     """:obj:`str`: 'MarkdownV2'"""
-    HTML = 'HTML'
+    HTML: str = 'HTML'
     """:obj:`str`: 'HTML'"""
