@@ -19,7 +19,7 @@
 try:
     import ujson as json
 except ImportError:
-    import json
+    import json  # type: ignore[no-redef]
 from base64 import b64decode
 
 from cryptography.hazmat.backends import default_backend
@@ -28,9 +28,13 @@ from cryptography.hazmat.primitives.ciphers import Cipher
 from cryptography.hazmat.primitives.ciphers.algorithms import AES
 from cryptography.hazmat.primitives.ciphers.modes import CBC
 from cryptography.hazmat.primitives.hashes import SHA512, SHA256, Hash, SHA1
-from future.utils import bord
 
 from telegram import TelegramObject, TelegramError
+from telegram.utils.types import JSONDict
+from typing import Union, Any, Optional, TYPE_CHECKING, List, no_type_check, Tuple
+
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class TelegramDecryptionError(TelegramError):
@@ -38,11 +42,15 @@ class TelegramDecryptionError(TelegramError):
     Something went wrong with decryption.
     """
 
-    def __init__(self, message):
-        super(TelegramDecryptionError, self).__init__("TelegramDecryptionError: "
-                                                      "{}".format(message))
+    def __init__(self, message: Union[str, Exception]):
+        super().__init__("TelegramDecryptionError: {}".format(message))
+        self._msg = str(message)
+
+    def __reduce__(self) -> Tuple[type, Tuple[str]]:
+        return self.__class__, (self._msg,)
 
 
+@no_type_check
 def decrypt(secret, hash, data):
     """
     Decrypt per telegram docs at https://core.telegram.org/passport.
@@ -83,9 +91,10 @@ def decrypt(secret, hash, data):
         # Raise a error that is caught inside telegram.PassportData and transformed into a warning
         raise TelegramDecryptionError("Hashes are not equal! {} != {}".format(data_hash, hash))
     # Return data without padding
-    return data[bord(data[0]):]
+    return data[data[0]:]
 
 
+@no_type_check
 def decrypt_json(secret, hash, data):
     """Decrypts data using secret and hash and then decodes utf-8 string and loads json"""
     return json.loads(decrypt(secret, hash, data).decode('utf-8'))
@@ -96,6 +105,9 @@ class EncryptedCredentials(TelegramObject):
     Telegram Passport Documentation for a complete description of the data decryption and
     authentication processes.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`data`, :attr:`hash` and :attr:`secret` are equal.
+
     Attributes:
         data (:class:`telegram.Credentials` or :obj:`str`): Decrypted data with unique user's
             nonce, data hashes and secrets used for EncryptedPassportElement decryption and
@@ -117,7 +129,12 @@ class EncryptedCredentials(TelegramObject):
 
     """
 
-    def __init__(self, data, hash, secret, bot=None, **kwargs):
+    def __init__(self,
+                 data: str,
+                 hash: str,
+                 secret: str,
+                 bot: 'Bot' = None,
+                 **kwargs: Any):
         # Required
         self.data = data
         self.hash = hash
@@ -127,19 +144,10 @@ class EncryptedCredentials(TelegramObject):
 
         self.bot = bot
         self._decrypted_secret = None
-        self._decrypted_data = None
-
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-
-        data = super(EncryptedCredentials, cls).de_json(data, bot)
-
-        return cls(bot=bot, **data)
+        self._decrypted_data: Optional['Credentials'] = None
 
     @property
-    def decrypted_secret(self):
+    def decrypted_secret(self) -> str:
         """
         :obj:`str`: Lazily decrypt and return secret.
 
@@ -166,7 +174,7 @@ class EncryptedCredentials(TelegramObject):
         return self._decrypted_secret
 
     @property
-    def decrypted_data(self):
+    def decrypted_data(self) -> 'Credentials':
         """
         :class:`telegram.Credentials`: Lazily decrypt and return credentials data. This object
             also contains the user specified nonce as
@@ -191,7 +199,7 @@ class Credentials(TelegramObject):
         nonce (:obj:`str`): Bot-specified nonce
     """
 
-    def __init__(self, secure_data, nonce, bot=None, **kwargs):
+    def __init__(self, secure_data: 'SecureData', nonce: str, bot: 'Bot' = None, **kwargs: Any):
         # Required
         self.secure_data = secure_data
         self.nonce = nonce
@@ -199,7 +207,9 @@ class Credentials(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['Credentials']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
@@ -237,19 +247,19 @@ class SecureData(TelegramObject):
     """
 
     def __init__(self,
-                 personal_details=None,
-                 passport=None,
-                 internal_passport=None,
-                 driver_license=None,
-                 identity_card=None,
-                 address=None,
-                 utility_bill=None,
-                 bank_statement=None,
-                 rental_agreement=None,
-                 passport_registration=None,
-                 temporary_registration=None,
-                 bot=None,
-                 **kwargs):
+                 personal_details: 'SecureValue' = None,
+                 passport: 'SecureValue' = None,
+                 internal_passport: 'SecureValue' = None,
+                 driver_license: 'SecureValue' = None,
+                 identity_card: 'SecureValue' = None,
+                 address: 'SecureValue' = None,
+                 utility_bill: 'SecureValue' = None,
+                 bank_statement: 'SecureValue' = None,
+                 rental_agreement: 'SecureValue' = None,
+                 passport_registration: 'SecureValue' = None,
+                 temporary_registration: 'SecureValue' = None,
+                 bot: 'Bot' = None,
+                 **kwargs: Any):
         # Optionals
         self.temporary_registration = temporary_registration
         self.passport_registration = passport_registration
@@ -266,7 +276,9 @@ class SecureData(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['SecureData']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
@@ -315,14 +327,14 @@ class SecureValue(TelegramObject):
     """
 
     def __init__(self,
-                 data=None,
-                 front_side=None,
-                 reverse_side=None,
-                 selfie=None,
-                 files=None,
-                 translation=None,
-                 bot=None,
-                 **kwargs):
+                 data: 'DataCredentials' = None,
+                 front_side: 'FileCredentials' = None,
+                 reverse_side: 'FileCredentials' = None,
+                 selfie: 'FileCredentials' = None,
+                 files: List['FileCredentials'] = None,
+                 translation: List['FileCredentials'] = None,
+                 bot: 'Bot' = None,
+                 **kwargs: Any):
         self.data = data
         self.front_side = front_side
         self.reverse_side = reverse_side
@@ -333,7 +345,9 @@ class SecureValue(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['SecureValue']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
@@ -346,8 +360,8 @@ class SecureValue(TelegramObject):
 
         return cls(bot=bot, **data)
 
-    def to_dict(self):
-        data = super(SecureValue, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         data['files'] = [p.to_dict() for p in self.files]
         data['translation'] = [p.to_dict() for p in self.translation]
@@ -358,7 +372,7 @@ class SecureValue(TelegramObject):
 class _CredentialsBase(TelegramObject):
     """Base class for DataCredentials and FileCredentials."""
 
-    def __init__(self, hash, secret, bot=None, **kwargs):
+    def __init__(self, hash: str, secret: str, bot: 'Bot' = None, **kwargs: Any):
         self.hash = hash
         self.secret = secret
 
@@ -368,24 +382,6 @@ class _CredentialsBase(TelegramObject):
 
         self.bot = bot
 
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-
-        return cls(bot=bot, **data)
-
-    @classmethod
-    def de_list(cls, data, bot):
-        if not data:
-            return []
-
-        credentials = list()
-        for c in data:
-            credentials.append(cls.de_json(c, bot=bot))
-
-        return credentials
-
 
 class DataCredentials(_CredentialsBase):
     """
@@ -401,11 +397,11 @@ class DataCredentials(_CredentialsBase):
         secret (:obj:`str`): Secret of encrypted data
     """
 
-    def __init__(self, data_hash, secret, **kwargs):
-        super(DataCredentials, self).__init__(data_hash, secret, **kwargs)
+    def __init__(self, data_hash: str, secret: str, **kwargs: Any):
+        super().__init__(data_hash, secret, **kwargs)
 
-    def to_dict(self):
-        data = super(DataCredentials, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         del data['file_hash']
         del data['hash']
@@ -427,11 +423,11 @@ class FileCredentials(_CredentialsBase):
             secret (:obj:`str`): Secret of encrypted file
         """
 
-    def __init__(self, file_hash, secret, **kwargs):
-        super(FileCredentials, self).__init__(file_hash, secret, **kwargs)
+    def __init__(self, file_hash: str, secret: str, **kwargs: Any):
+        super().__init__(file_hash, secret, **kwargs)
 
-    def to_dict(self):
-        data = super(FileCredentials, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         del data['data_hash']
         del data['hash']
