@@ -19,16 +19,25 @@
 """This module contains an object that represents a Telegram EncryptedPassportElement."""
 from base64 import b64decode
 
-from telegram import (TelegramObject, PassportFile, PersonalDetails, IdDocumentData,
-                      ResidentialAddress)
+from telegram import (IdDocumentData, PassportFile, PersonalDetails,
+                      ResidentialAddress, TelegramObject)
 from telegram.passport.credentials import decrypt_json
 
+from telegram.utils.types import JSONDict
+from typing import List, Any, Optional, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot, Credentials
+
 
 class EncryptedPassportElement(TelegramObject):
     """
     Contains information about documents or other Telegram Passport elements shared with the bot
     by the user. The data has been automatically decrypted by python-telegram-bot.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`type`, :attr:`data`, :attr:`phone_number`, :attr:`email`,
+    :attr:`files`, :attr:`front_side`, :attr:`reverse_side` and :attr:`selfie` are equal.
+
     Attributes:
         type (:obj:`str`): Element type. One of "personal_details", "passport", "driver_license",
             "identity_card", "internal_passport", "address", "utility_bill", "bank_statement",
@@ -102,19 +111,19 @@ class EncryptedPassportElement(TelegramObject):
     """
 
     def __init__(self,
-                 type,
-                 data=None,
-                 phone_number=None,
-                 email=None,
-                 files=None,
-                 front_side=None,
-                 reverse_side=None,
-                 selfie=None,
-                 translation=None,
-                 hash=None,
-                 bot=None,
-                 credentials=None,
-                 **kwargs):
+                 type: str,
+                 data: PersonalDetails = None,
+                 phone_number: str = None,
+                 email: str = None,
+                 files: List[PassportFile] = None,
+                 front_side: PassportFile = None,
+                 reverse_side: PassportFile = None,
+                 selfie: PassportFile = None,
+                 translation: List[PassportFile] = None,
+                 hash: str = None,
+                 bot: 'Bot' = None,
+                 credentials: 'Credentials' = None,
+                 **kwargs: Any):
         # Required
         self.type = type
         # Optionals
@@ -134,12 +143,14 @@ class EncryptedPassportElement(TelegramObject):
         self.bot = bot
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls,
+                data: Optional[JSONDict],
+                bot: 'Bot') -> Optional['EncryptedPassportElement']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super(EncryptedPassportElement, cls).de_json(data, bot)
-
         data['files'] = PassportFile.de_list(data.get('files'), bot) or None
         data['front_side'] = PassportFile.de_json(data.get('front_side'), bot)
         data['reverse_side'] = PassportFile.de_json(data.get('reverse_side'), bot)
@@ -149,12 +160,13 @@ class EncryptedPassportElement(TelegramObject):
         return cls(bot=bot, **data)
 
     @classmethod
-    def de_json_decrypted(cls, data, bot, credentials):
+    def de_json_decrypted(cls,
+                          data: Optional[JSONDict],
+                          bot: 'Bot',
+                          credentials: 'Credentials') -> Optional['EncryptedPassportElement']:
         if not data:
             return None
 
-        data = super(EncryptedPassportElement, cls).de_json(data, bot)
-
         if data['type'] not in ('phone_number', 'email'):
             secure_data = getattr(credentials.secure_data, data['type'])
 
@@ -185,19 +197,8 @@ class EncryptedPassportElement(TelegramObject):
 
         return cls(bot=bot, **data)
 
-    @classmethod
-    def de_list(cls, data, bot):
-        if not data:
-            return []
-
-        encrypted_passport_elements = list()
-        for element in data:
-            encrypted_passport_elements.append(cls.de_json(element, bot))
-
-        return encrypted_passport_elements
-
-    def to_dict(self):
-        data = super(EncryptedPassportElement, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         if self.files:
             data['files'] = [p.to_dict() for p in self.files]
