@@ -19,11 +19,19 @@
 """This module contains an object that represents a Telegram OrderInfo."""
 
 from telegram import TelegramObject, ShippingAddress
+from telegram.utils.types import JSONDict
+from typing import Any, Optional, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class OrderInfo(TelegramObject):
     """This object represents information about an order.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`name`, :attr:`phone_number`, :attr:`email` and
+    :attr:`shipping_address` are equal.
+
     Attributes:
         name (:obj:`str`): Optional. User name.
         phone_number (:obj:`str`): Optional. User's phone number.
@@ -39,19 +47,26 @@ class OrderInfo(TelegramObject):
 
     """
 
-    def __init__(self, name=None, phone_number=None, email=None, shipping_address=None, **kwargs):
+    def __init__(self,
+                 name: str = None,
+                 phone_number: str = None,
+                 email: str = None,
+                 shipping_address: str = None,
+                 **kwargs: Any):
         self.name = name
         self.phone_number = phone_number
         self.email = email
         self.shipping_address = shipping_address
 
+        self._id_attrs = (self.name, self.phone_number, self.email, self.shipping_address)
+
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['OrderInfo']:
+        data = cls.parse_data(data)
+
         if not data:
             return cls()
 
-        data = super(OrderInfo, cls).de_json(data, bot)
-
         data['shipping_address'] = ShippingAddress.de_json(data.get('shipping_address'), bot)
 
         return cls(**data)
