@@ -19,11 +19,18 @@
 """This module contains an object that represents a Telegram ShippingQuery."""
 
 from telegram import TelegramObject, User, ShippingAddress
+from telegram.utils.types import JSONDict
+from typing import Any, Optional, TYPE_CHECKING
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class ShippingQuery(TelegramObject):
     """This object contains information about an incoming shipping query.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`id` is equal.
+
     Note:
         * In Python `from` is a reserved word, use `from_user` instead.
 
@@ -44,7 +51,13 @@ class ShippingQuery(TelegramObject):
 
     """
 
-    def __init__(self, id, from_user, invoice_payload, shipping_address, bot=None, **kwargs):
+    def __init__(self,
+                 id: str,
+                 from_user: User,
+                 invoice_payload: str,
+                 shipping_address: ShippingAddress,
+                 bot: 'Bot' = None,
+                 **kwargs: Any):
         self.id = id
         self.from_user = from_user
         self.invoice_payload = invoice_payload
@@ -55,30 +68,30 @@ class ShippingQuery(TelegramObject):
         self._id_attrs = (self.id,)
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['ShippingQuery']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super(ShippingQuery, cls).de_json(data, bot)
-
         data['from_user'] = User.de_json(data.pop('from'), bot)
         data['shipping_address'] = ShippingAddress.de_json(data.get('shipping_address'), bot)
 
         return cls(bot=bot, **data)
 
-    def answer(self, *args, **kwargs):
+    def answer(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
             bot.answer_shipping_query(update.shipping_query.id, *args, **kwargs)
 
         Args:
-            ok (:obj:`bool`): Specify True if delivery to the specified address is possible and
-                False if there are any problems (for example, if delivery to the specified address
-                is not possible).
+            ok (:obj:`bool`): Specify :obj:`True` if delivery to the specified address is
+                possible and :obj:`False` if there are any problems
+                (for example, if delivery to the specified address is not possible).
             shipping_options (List[:class:`telegram.ShippingOption`], optional): Required if ok is
-                True. A JSON-serialized array of available shipping options.
-            error_message (:obj:`str`, optional): Required if ok is False. Error message in human
-                readable form that explains why it is impossible to complete the order (e.g.
+                :obj:`True`. A JSON-serialized array of available shipping options.
+            error_message (:obj:`str`, optional): Required if ok is :obj:`False`. Error message in
+                human readable form that explains why it is impossible to complete the order (e.g.
                 "Sorry, delivery to your desired address is unavailable'). Telegram will display
                 this message to the user.
 
