@@ -20,15 +20,24 @@
 """This module contains an object that represents a Telegram Poll."""
 
 import sys
+import datetime
 
 from telegram import (TelegramObject, User, MessageEntity)
 from telegram.utils.helpers import to_timestamp, from_timestamp
+from telegram.utils.types import JSONDict
+from typing import Any, Dict, Optional, List, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class PollOption(TelegramObject):
     """
     This object contains information about one answer option in a poll.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`text` and :attr:`voter_count` are equal.
+
     Attributes:
         text (:obj:`str`): Option text, 1-100 characters.
         voter_count (:obj:`int`): Number of users that voted for this option.
@@ -39,22 +48,20 @@ class PollOption(TelegramObject):
 
     """
 
-    def __init__(self, text, voter_count, **kwargs):
+    def __init__(self, text: str, voter_count: int, **kwargs: Any):
         self.text = text
         self.voter_count = voter_count
 
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-
-        return cls(**data)
+        self._id_attrs = (self.text, self.voter_count)
 
 
 class PollAnswer(TelegramObject):
     """
     This object represents an answer of a user in a non-anonymous poll.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`poll_id`, :attr:`user` and :attr:`options_ids` are equal.
+
     Attributes:
         poll_id (:obj:`str`): Unique poll identifier.
         user (:class:`telegram.User`): The user, who changed the answer to the poll.
@@ -67,18 +74,20 @@ class PollAnswer(TelegramObject):
             May be empty if the user retracted their vote.
 
     """
-    def __init__(self, poll_id, user, option_ids, **kwargs):
+    def __init__(self, poll_id: str, user: User, option_ids: List[int], **kwargs: Any):
         self.poll_id = poll_id
         self.user = user
         self.option_ids = option_ids
 
+        self._id_attrs = (self.poll_id, self.user, tuple(self.option_ids))
+
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['PollAnswer']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super(PollAnswer, cls).de_json(data, bot)
-
         data['user'] = User.de_json(data.get('user'), bot)
 
         return cls(**data)
@@ -88,15 +97,18 @@ class Poll(TelegramObject):
     """
     This object contains information about a poll.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`id` is equal.
+
     Attributes:
         id (:obj:`str`): Unique poll identifier.
         question (:obj:`str`): Poll question, 1-255 characters.
         options (List[:class:`PollOption`]): List of poll options.
         total_voter_count (:obj:`int`): Total number of users that voted in the poll.
-        is_closed (:obj:`bool`): True, if the poll is closed.
-        is_anonymous (:obj:`bool`): True, if the poll is anonymous.
+        is_closed (:obj:`bool`): :obj:`True`, if the poll is closed.
+        is_anonymous (:obj:`bool`): :obj:`True`, if the poll is anonymous.
         type (:obj:`str`): Poll type, currently can be :attr:`REGULAR` or :attr:`QUIZ`.
-        allows_multiple_answers (:obj:`bool`): True, if the poll allows multiple answers.
+        allows_multiple_answers (:obj:`bool`): :obj:`True`, if the poll allows multiple answers.
         correct_option_id (:obj:`int`): Optional. Identifier of the correct answer option.
         explanation (:obj:`str`): Optional. Text that is shown when a user chooses an incorrect
             answer or taps on the lamp icon in a quiz-style poll.
@@ -111,10 +123,10 @@ class Poll(TelegramObject):
         id (:obj:`str`): Unique poll identifier.
         question (:obj:`str`): Poll question, 1-255 characters.
         options (List[:class:`PollOption`]): List of poll options.
-        is_closed (:obj:`bool`): True, if the poll is closed.
-        is_anonymous (:obj:`bool`): True, if the poll is anonymous.
+        is_closed (:obj:`bool`): :obj:`True`, if the poll is closed.
+        is_anonymous (:obj:`bool`): :obj:`True`, if the poll is anonymous.
         type (:obj:`str`): Poll type, currently can be :attr:`REGULAR` or :attr:`QUIZ`.
-        allows_multiple_answers (:obj:`bool`): True, if the poll allows multiple answers.
+        allows_multiple_answers (:obj:`bool`): :obj:`True`, if the poll allows multiple answers.
         correct_option_id (:obj:`int`, optional): 0-based identifier of the correct answer option.
             Available only for polls in the quiz mode, which are closed, or was sent (not
             forwarded) by the bot or to the private chat with the bot.
@@ -130,20 +142,20 @@ class Poll(TelegramObject):
     """
 
     def __init__(self,
-                 id,
-                 question,
-                 options,
-                 total_voter_count,
-                 is_closed,
-                 is_anonymous,
-                 type,
-                 allows_multiple_answers,
-                 correct_option_id=None,
-                 explanation=None,
-                 explanation_entities=None,
-                 open_period=None,
-                 close_date=None,
-                 **kwargs):
+                 id: str,
+                 question: str,
+                 options: List[PollOption],
+                 total_voter_count: int,
+                 is_closed: bool,
+                 is_anonymous: bool,
+                 type: str,
+                 allows_multiple_answers: bool,
+                 correct_option_id: int = None,
+                 explanation: str = None,
+                 explanation_entities: List[MessageEntity] = None,
+                 open_period: int = None,
+                 close_date: datetime.datetime = None,
+                 **kwargs: Any):
         self.id = id
         self.question = question
         self.options = options
@@ -161,19 +173,19 @@ class Poll(TelegramObject):
         self._id_attrs = (self.id,)
 
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['Poll']:
+        data = cls.parse_data(data)
+
         if not data:
             return None
 
-        data = super().de_json(data, bot)
-
         data['options'] = [PollOption.de_json(option, bot) for option in data['options']]
         data['explanation_entities'] = MessageEntity.de_list(data.get('explanation_entities'), bot)
         data['close_date'] = from_timestamp(data.get('close_date'))
 
         return cls(**data)
 
-    def to_dict(self):
+    def to_dict(self) -> JSONDict:
         data = super().to_dict()
 
         data['options'] = [x.to_dict() for x in self.options]
@@ -183,7 +195,7 @@ class Poll(TelegramObject):
 
         return data
 
-    def parse_explanation_entity(self, entity):
+    def parse_explanation_entity(self, entity: MessageEntity) -> str:
         """Returns the text from a given :class:`telegram.MessageEntity`.
 
         Note:
@@ -198,7 +210,13 @@ class Poll(TelegramObject):
         Returns:
             :obj:`str`: The text of the given entity.
 
+        Raises:
+            RuntimeError: If the poll has no explanation.
+
         """
+        if not self.explanation:
+            raise RuntimeError("This Poll has no 'explanation'.")
+
         # Is it a narrow build, if so we don't need to convert
         if sys.maxunicode == 0xffff:
             return self.explanation[entity.offset:entity.offset + entity.length]
@@ -208,7 +226,7 @@ class Poll(TelegramObject):
 
         return entity_text.decode('utf-16-le')
 
-    def parse_explanation_entities(self, types=None):
+    def parse_explanation_entities(self, types: List[str] = None) -> Dict[MessageEntity, str]:
         """
         Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
         It contains entities from this polls explanation filtered by their ``type`` attribute as
@@ -234,10 +252,10 @@ class Poll(TelegramObject):
 
         return {
             entity: self.parse_explanation_entity(entity)
-            for entity in self.explanation_entities if entity.type in types
+            for entity in (self.explanation_entities or []) if entity.type in types
         }
 
-    REGULAR = "regular"
+    REGULAR: str = "regular"
     """:obj:`str`: 'regular'"""
-    QUIZ = "quiz"
+    QUIZ: str = "quiz"
     """:obj:`str`: 'quiz'"""
