@@ -18,12 +18,17 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram ReplyKeyboardMarkup."""
 
-from telegram import ReplyMarkup
+from telegram import ReplyMarkup, KeyboardButton
+from telegram.utils.types import JSONDict
+from typing import List, Union, Any
 
 
 class ReplyKeyboardMarkup(ReplyMarkup):
     """This object represents a custom keyboard with reply options.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their the size of :attr:`keyboard` and all the buttons are equal.
+
     Attributes:
         keyboard (List[List[:class:`telegram.KeyboardButton` | :obj:`str`]]): Array of button rows.
         resize_keyboard (:obj:`bool`): Optional. Requests clients to resize the keyboard.
@@ -40,46 +45,55 @@ class ReplyKeyboardMarkup(ReplyMarkup):
                 each represented by an Array of :class:`telegram.KeyboardButton` objects.
         resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard vertically
             for optimal fit (e.g., make the keyboard smaller if there are just two rows of
-            buttons). Defaults to false, in which case the custom keyboard is always of the same
-            height as the app's standard keyboard. Defaults to ``False``
+            buttons). Defaults to :obj:`False`, in which case the custom keyboard is always of the
+            same height as the app's standard keyboard.
         one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as soon as
             it's been used. The keyboard will still be available, but clients will automatically
             display the usual letter-keyboard in the chat - the user can press a special button in
-            the input field to see the custom keyboard again. Defaults to ``False``.
+            the input field to see the custom keyboard again. Defaults to :obj:`False`.
         selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard to
             specific users only. Targets:
 
-            1) users that are @mentioned in the text of the Message object
-            2) if the bot's message is a reply (has reply_to_message_id), sender of the original
-               message.
+            1) Users that are @mentioned in the text of the Message object.
+            2) If the bot's message is a reply (has ``reply_to_message_id``), sender of the
+               original message.
 
-            Defaults to ``False``.
+            Defaults to :obj:`False`.
 
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
     """
 
     def __init__(self,
-                 keyboard,
-                 resize_keyboard=False,
-                 one_time_keyboard=False,
-                 selective=False,
-                 **kwargs):
+                 keyboard: List[List[Union[str, KeyboardButton]]],
+                 resize_keyboard: bool = False,
+                 one_time_keyboard: bool = False,
+                 selective: bool = False,
+                 **kwargs: Any):
         # Required
-        self.keyboard = keyboard
+        self.keyboard = []
+        for row in keyboard:
+            r = []
+            for button in row:
+                if isinstance(button, KeyboardButton):
+                    r.append(button)  # telegram.KeyboardButton
+                else:
+                    r.append(KeyboardButton(button))  # str
+            self.keyboard.append(r)
+
         # Optionals
         self.resize_keyboard = bool(resize_keyboard)
         self.one_time_keyboard = bool(one_time_keyboard)
         self.selective = bool(selective)
 
-    def to_dict(self):
-        data = super(ReplyKeyboardMarkup, self).to_dict()
+    def to_dict(self) -> JSONDict:
+        data = super().to_dict()
 
         data['keyboard'] = []
         for row in self.keyboard:
-            r = []
+            r: List[Union[JSONDict, str]] = []
             for button in row:
-                if hasattr(button, 'to_dict'):
+                if isinstance(button, KeyboardButton):
                     r.append(button.to_dict())  # telegram.KeyboardButton
                 else:
                     r.append(button)  # str
@@ -88,37 +102,37 @@ class ReplyKeyboardMarkup(ReplyMarkup):
 
     @classmethod
     def from_button(cls,
-                    button,
-                    resize_keyboard=False,
-                    one_time_keyboard=False,
-                    selective=False,
-                    **kwargs):
+                    button: Union[KeyboardButton, str],
+                    resize_keyboard: bool = False,
+                    one_time_keyboard: bool = False,
+                    selective: bool = False,
+                    **kwargs: Any) -> 'ReplyKeyboardMarkup':
         """Shortcut for::
 
             ReplyKeyboardMarkup([[button]], **kwargs)
 
-        Return an ReplyKeyboardMarkup from a single KeyboardButton
+        Return a ReplyKeyboardMarkup from a single KeyboardButton.
 
         Args:
-            button (:class:`telegram.KeyboardButton` | :obj:`str`): The button to use in the markup
+            button (:class:`telegram.KeyboardButton` | :obj:`str`): The button to use in
+                the markup.
             resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                 vertically for optimal fit (e.g., make the keyboard smaller if there are just two
-                rows of buttons). Defaults to false, in which case the custom keyboard is always of
-                the same height as the app's standard keyboard.
-                Defaults to ``False``
+                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
+                always of the same height as the app's standard keyboard.
             one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                 soon as it's been used. The keyboard will still be available, but clients will
                 automatically display the usual letter-keyboard in the chat - the user can press
                 a special button in the input field to see the custom keyboard again.
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                 to specific users only. Targets:
 
-                1) users that are @mentioned in the text of the Message object
-                2) if the bot's message is a reply (has reply_to_message_id), sender of the
+                1) Users that are @mentioned in the text of the Message object.
+                2) If the bot's message is a reply (has reply_to_message_id), sender of the
                     original message.
 
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             **kwargs (:obj:`dict`): Arbitrary keyword arguments.
         """
         return cls([[button]],
@@ -129,38 +143,37 @@ class ReplyKeyboardMarkup(ReplyMarkup):
 
     @classmethod
     def from_row(cls,
-                 button_row,
-                 resize_keyboard=False,
-                 one_time_keyboard=False,
-                 selective=False,
-                 **kwargs):
+                 button_row: List[Union[str, KeyboardButton]],
+                 resize_keyboard: bool = False,
+                 one_time_keyboard: bool = False,
+                 selective: bool = False,
+                 **kwargs: Any) -> 'ReplyKeyboardMarkup':
         """Shortcut for::
 
             ReplyKeyboardMarkup([button_row], **kwargs)
 
-        Return an ReplyKeyboardMarkup from a single row of KeyboardButtons
+        Return a ReplyKeyboardMarkup from a single row of KeyboardButtons.
 
         Args:
             button_row (List[:class:`telegram.KeyboardButton` | :obj:`str`]): The button to use in
-                the markup
+                the markup.
             resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                 vertically for optimal fit (e.g., make the keyboard smaller if there are just two
-                rows of buttons). Defaults to false, in which case the custom keyboard is always of
-                the same height as the app's standard keyboard.
-                Defaults to ``False``
+                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
+                always of the same height as the app's standard keyboard.
             one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                 soon as it's been used. The keyboard will still be available, but clients will
                 automatically display the usual letter-keyboard in the chat - the user can press
                 a special button in the input field to see the custom keyboard again.
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                 to specific users only. Targets:
 
-                1) users that are @mentioned in the text of the Message object
-                2) if the bot's message is a reply (has reply_to_message_id), sender of the
+                1) Users that are @mentioned in the text of the Message object.
+                2) If the bot's message is a reply (has reply_to_message_id), sender of the
                     original message.
 
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
         """
@@ -172,38 +185,37 @@ class ReplyKeyboardMarkup(ReplyMarkup):
 
     @classmethod
     def from_column(cls,
-                    button_column,
-                    resize_keyboard=False,
-                    one_time_keyboard=False,
-                    selective=False,
-                    **kwargs):
+                    button_column: List[Union[str, KeyboardButton]],
+                    resize_keyboard: bool = False,
+                    one_time_keyboard: bool = False,
+                    selective: bool = False,
+                    **kwargs: Any) -> 'ReplyKeyboardMarkup':
         """Shortcut for::
 
             ReplyKeyboardMarkup([[button] for button in button_column], **kwargs)
 
-        Return an ReplyKeyboardMarkup from a single column of KeyboardButtons
+        Return a ReplyKeyboardMarkup from a single column of KeyboardButtons.
 
         Args:
             button_column (List[:class:`telegram.KeyboardButton` | :obj:`str`]): The button to use
-                in the markup
+                in the markup.
             resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                 vertically for optimal fit (e.g., make the keyboard smaller if there are just two
-                rows of buttons). Defaults to false, in which case the custom keyboard is always of
-                the same height as the app's standard keyboard.
-                Defaults to ``False``
+                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
+                always of the same height as the app's standard keyboard.
             one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                 soon as it's been used. The keyboard will still be available, but clients will
                 automatically display the usual letter-keyboard in the chat - the user can press
                 a special button in the input field to see the custom keyboard again.
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                 to specific users only. Targets:
 
-                1) users that are @mentioned in the text of the Message object
-                2) if the bot's message is a reply (has reply_to_message_id), sender of the
+                1) Users that are @mentioned in the text of the Message object.
+                2) If the bot's message is a reply (has reply_to_message_id), sender of the
                     original message.
 
-                Defaults to ``False``.
+                Defaults to :obj:`False`.
             **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
         """
@@ -213,3 +225,22 @@ class ReplyKeyboardMarkup(ReplyMarkup):
                    one_time_keyboard=one_time_keyboard,
                    selective=selective,
                    **kwargs)
+
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, self.__class__):
+            if len(self.keyboard) != len(other.keyboard):
+                return False
+            for idx, row in enumerate(self.keyboard):
+                if len(row) != len(other.keyboard[idx]):
+                    return False
+                for jdx, button in enumerate(row):
+                    if button != other.keyboard[idx][jdx]:
+                        return False
+            return True
+        return super(ReplyKeyboardMarkup, self).__eq__(other)  # pylint: disable=no-member
+
+    def __hash__(self) -> int:
+        return hash((
+            tuple(tuple(button for button in row) for row in self.keyboard),
+            self.resize_keyboard, self.one_time_keyboard, self.selective
+        ))
