@@ -23,54 +23,62 @@ from telegram import TelegramObject
 from telegram.utils.helpers import mention_html as util_mention_html
 from telegram.utils.helpers import mention_markdown as util_mention_markdown
 
+from typing import Any, Optional, TYPE_CHECKING, List
+
+if TYPE_CHECKING:
+    from telegram import Bot, UserProfilePhotos, Message
+
 
 class User(TelegramObject):
     """This object represents a Telegram user or bot.
 
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`id` is equal.
+
     Attributes:
         id (:obj:`int`): Unique identifier for this user or bot.
-        is_bot (:obj:`bool`): True, if this user is a bot
+        is_bot (:obj:`bool`): :obj:`True`, if this user is a bot.
         first_name (:obj:`str`): User's or bot's first name.
         last_name (:obj:`str`): Optional. User's or bot's last name.
         username (:obj:`str`): Optional. User's or bot's username.
         language_code (:obj:`str`): Optional. IETF language tag of the user's language.
-        can_join_groups (:obj:`str`): Optional. True, if the bot can be invited to groups.
-            Returned only in :attr:`telegram.Bot.get_me` requests.
-        can_read_all_group_messages (:obj:`str`): Optional. True, if privacy mode is disabled
-            for the bot. Returned only in :attr:`telegram.Bot.get_me` requests.
-        supports_inline_queries (:obj:`str`): Optional. True, if the bot supports inline queries.
+        can_join_groups (:obj:`str`): Optional. :obj:`True`, if the bot can be invited to groups.
             Returned only in :attr:`telegram.Bot.get_me` requests.
+        can_read_all_group_messages (:obj:`str`): Optional. :obj:`True`, if privacy mode is
+            disabled for the bot. Returned only in :attr:`telegram.Bot.get_me` requests.
+        supports_inline_queries (:obj:`str`): Optional. :obj:`True`, if the bot supports inline
+            queries. Returned only in :attr:`telegram.Bot.get_me` requests.
         bot (:class:`telegram.Bot`): Optional. The Bot to use for instance methods.
 
     Args:
         id (:obj:`int`): Unique identifier for this user or bot.
-        is_bot (:obj:`bool`): True, if this user is a bot
+        is_bot (:obj:`bool`): :obj:`True`, if this user is a bot.
         first_name (:obj:`str`): User's or bot's first name.
         last_name (:obj:`str`, optional): User's or bot's last name.
         username (:obj:`str`, optional): User's or bot's username.
         language_code (:obj:`str`, optional): IETF language tag of the user's language.
-        can_join_groups (:obj:`str`, optional): True, if the bot can be invited to groups.
-            Returned only in :attr:`telegram.Bot.get_me` requests.
-        can_read_all_group_messages (:obj:`str`, optional): True, if privacy mode is disabled
-            for the bot. Returned only in :attr:`telegram.Bot.get_me` requests.
-        supports_inline_queries (:obj:`str`, optional): True, if the bot supports inline queries.
+        can_join_groups (:obj:`str`, optional): :obj:`True`, if the bot can be invited to groups.
             Returned only in :attr:`telegram.Bot.get_me` requests.
+        can_read_all_group_messages (:obj:`str`, optional): :obj:`True`, if privacy mode is
+            disabled for the bot. Returned only in :attr:`telegram.Bot.get_me` requests.
+        supports_inline_queries (:obj:`str`, optional): :obj:`True`, if the bot supports inline
+            queries. Returned only in :attr:`telegram.Bot.get_me` requests.
         bot (:class:`telegram.Bot`, optional): The Bot to use for instance methods.
 
     """
 
     def __init__(self,
-                 id,
-                 first_name,
-                 is_bot,
-                 last_name=None,
-                 username=None,
-                 language_code=None,
-                 can_join_groups=None,
-                 can_read_all_group_messages=None,
-                 supports_inline_queries=None,
-                 bot=None,
-                 **kwargs):
+                 id: int,
+                 first_name: str,
+                 is_bot: bool,
+                 last_name: str = None,
+                 username: str = None,
+                 language_code: str = None,
+                 can_join_groups: bool = None,
+                 can_read_all_group_messages: bool = None,
+                 supports_inline_queries: bool = None,
+                 bot: 'Bot' = None,
+                 **kwargs: Any):
         # Required
         self.id = int(id)
         self.first_name = first_name
@@ -87,7 +95,7 @@ class User(TelegramObject):
         self._id_attrs = (self.id,)
 
     @property
-    def name(self):
+    def name(self) -> str:
         """:obj:`str`: Convenience property. If available, returns the user's :attr:`username`
         prefixed with "@". If :attr:`username` is not available, returns :attr:`full_name`."""
         if self.username:
@@ -95,7 +103,7 @@ class User(TelegramObject):
         return self.full_name
 
     @property
-    def full_name(self):
+    def full_name(self) -> str:
         """:obj:`str`: Convenience property. The user's :attr:`first_name`, followed by (if
         available) :attr:`last_name`."""
 
@@ -104,7 +112,7 @@ class User(TelegramObject):
         return self.first_name
 
     @property
-    def link(self):
+    def link(self) -> Optional[str]:
         """:obj:`str`: Convenience property. If :attr:`username` is available, returns a t.me link
         of the user."""
 
@@ -112,37 +120,22 @@ class User(TelegramObject):
             return "https://t.me/{}".format(self.username)
         return None
 
-    @classmethod
-    def de_json(cls, data, bot):
-        if not data:
-            return None
-        data = super().de_json(data, bot)
-
-        return cls(bot=bot, **data)
-
-    def get_profile_photos(self, *args, **kwargs):
+    def get_profile_photos(self, *args: Any, **kwargs: Any) -> 'UserProfilePhotos':
         """
         Shortcut for::
 
-                bot.get_user_profile_photos(update.message.from_user.id, *args, **kwargs)
+                bot.get_user_profile_photos(update.effective_user.id, *args, **kwargs)
 
         """
 
         return self.bot.get_user_profile_photos(self.id, *args, **kwargs)
 
-    @classmethod
-    def de_list(cls, data, bot):
-        if not data:
-            return []
-
-        users = list()
-        for user in data:
-            users.append(cls.de_json(user, bot))
-
-        return users
-
-    def mention_markdown(self, name=None):
+    def mention_markdown(self, name: str = None) -> str:
         """
+        Note:
+            :attr:`telegram.ParseMode.MARKDOWN` is is a legacy mode, retained by Telegram for
+            backward compatibility. You should use :meth:`mention_markdown_v2` instead.
+
         Args:
             name (:obj:`str`): The name used as a link for the user. Defaults to :attr:`full_name`.
 
@@ -154,7 +147,7 @@ class User(TelegramObject):
             return util_mention_markdown(self.id, name)
         return util_mention_markdown(self.id, self.full_name)
 
-    def mention_markdown_v2(self, name=None):
+    def mention_markdown_v2(self, name: str = None) -> str:
         """
         Args:
             name (:obj:`str`): The name used as a link for the user. Defaults to :attr:`full_name`.
@@ -167,7 +160,7 @@ class User(TelegramObject):
             return util_mention_markdown(self.id, name, version=2)
         return util_mention_markdown(self.id, self.full_name, version=2)
 
-    def mention_html(self, name=None):
+    def mention_html(self, name: str = None) -> str:
         """
         Args:
             name (:obj:`str`): The name used as a link for the user. Defaults to :attr:`full_name`.
@@ -180,12 +173,10 @@ class User(TelegramObject):
             return util_mention_html(self.id, name)
         return util_mention_html(self.id, self.full_name)
 
-    def send_message(self, *args, **kwargs):
+    def send_message(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_message(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_message(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -193,12 +184,10 @@ class User(TelegramObject):
         """
         return self.bot.send_message(self.id, *args, **kwargs)
 
-    def send_photo(self, *args, **kwargs):
+    def send_photo(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_photo(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_photo(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -206,12 +195,21 @@ class User(TelegramObject):
         """
         return self.bot.send_photo(self.id, *args, **kwargs)
 
-    def send_audio(self, *args, **kwargs):
+    def send_media_group(self, *args: Any, **kwargs: Any) -> List['Message']:
         """Shortcut for::
 
-            bot.send_audio(User.id, *args, **kwargs)
+            bot.send_media_group(update.effective_user.id, *args, **kwargs)
 
-        Where User is the current instance.
+        Returns:
+            List[:class:`telegram.Message`:] On success, instance representing the message posted.
+
+        """
+        return self.bot.send_media_group(self.id, *args, **kwargs)
+
+    def send_audio(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_audio(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -219,12 +217,46 @@ class User(TelegramObject):
         """
         return self.bot.send_audio(self.id, *args, **kwargs)
 
-    def send_document(self, *args, **kwargs):
+    def send_chat_action(self, *args: Any, **kwargs: Any) -> bool:
         """Shortcut for::
 
-            bot.send_document(User.id, *args, **kwargs)
+            bot.send_chat_action(update.effective_user.id, *args, **kwargs)
 
-        Where User is the current instance.
+        Returns:
+            :obj:`True`: On success.
+
+        """
+        return self.bot.send_chat_action(self.id, *args, **kwargs)
+
+    send_action = send_chat_action
+    """Alias for :attr:`send_chat_action`"""
+
+    def send_contact(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_contact(update.effective_user.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_contact(self.id, *args, **kwargs)
+
+    def send_dice(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_dice(update.effective_user.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_dice(self.id, *args, **kwargs)
+
+    def send_document(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_document(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -232,12 +264,43 @@ class User(TelegramObject):
         """
         return self.bot.send_document(self.id, *args, **kwargs)
 
-    def send_animation(self, *args, **kwargs):
+    def send_game(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_animation(User.id, *args, **kwargs)
+            bot.send_game(update.effective_user.id, *args, **kwargs)
 
-        Where User is the current instance.
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_game(self.id, *args, **kwargs)
+
+    def send_invoice(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_invoice(update.effective_user.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_invoice(self.id, *args, **kwargs)
+
+    def send_location(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_location(update.effective_user.id, *args, **kwargs)
+
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_location(self.id, *args, **kwargs)
+
+    def send_animation(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_animation(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -245,12 +308,10 @@ class User(TelegramObject):
         """
         return self.bot.send_animation(self.id, *args, **kwargs)
 
-    def send_sticker(self, *args, **kwargs):
+    def send_sticker(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_sticker(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_sticker(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -258,12 +319,10 @@ class User(TelegramObject):
         """
         return self.bot.send_sticker(self.id, *args, **kwargs)
 
-    def send_video(self, *args, **kwargs):
+    def send_video(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_video(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_video(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -271,12 +330,21 @@ class User(TelegramObject):
         """
         return self.bot.send_video(self.id, *args, **kwargs)
 
-    def send_video_note(self, *args, **kwargs):
+    def send_venue(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_video_note(User.id, *args, **kwargs)
+            bot.send_venue(update.effective_user.id, *args, **kwargs)
 
-        Where User is the current instance.
+        Returns:
+            :class:`telegram.Message`: On success, instance representing the message posted.
+
+        """
+        return self.bot.send_venue(self.id, *args, **kwargs)
+
+    def send_video_note(self, *args: Any, **kwargs: Any) -> 'Message':
+        """Shortcut for::
+
+            bot.send_video_note(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -284,12 +352,10 @@ class User(TelegramObject):
         """
         return self.bot.send_video_note(self.id, *args, **kwargs)
 
-    def send_voice(self, *args, **kwargs):
+    def send_voice(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_voice(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_voice(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
@@ -297,12 +363,10 @@ class User(TelegramObject):
         """
         return self.bot.send_voice(self.id, *args, **kwargs)
 
-    def send_poll(self, *args, **kwargs):
+    def send_poll(self, *args: Any, **kwargs: Any) -> 'Message':
         """Shortcut for::
 
-            bot.send_poll(User.id, *args, **kwargs)
-
-        Where User is the current instance.
+            bot.send_poll(update.effective_user.id, *args, **kwargs)
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the message posted.
